package lopsiltest.app3policy;
import lopsil.device.Location;
import lopsil.device.Position;
import lopsil.Action;
import lopsil.LopsilException;
import lopsil.Reaction;
import android.app.Activity;
import android.content.Context;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;

// the aspect class allows for inlining code into compiled java bitcode.
public aspect Aspect extends Activity implements LocationListener {
	private ShowNavigationAid p;        // the policy to be enforced
	private LocationManager locman;		// needed to observe location updates in android
	private Activity _act;				// needed to interact with interface of android app

	// creates lopsil policy only after application creation
	after(): execution(* *.onCreate(..)) {	
		_act = (Activity)thisJoinPoint.getThis();
	    p = new ShowNavigationAid(_act);	    
	   
	    // initialize own location manager in order to update policy
		locman = (LocationManager) _act.getSystemService(Context.LOCATION_SERVICE);
	    locman.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);	
	    locman.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, this);	
	}	

	// method to take the incoming location update and set the 
	// location device of the policy
	@Override
	public void onLocationChanged(android.location.Location loc) {
		Position pos = new Position(loc.getLatitude(), loc.getLongitude());
    	Location l = new Location(pos);
    	p.devices[0].setLocation(l);
	}
	
	// pointcuts to specify where to inline policy
	pointcut notInAdvice():
		!cflow(adviceexecution()) && !within(ShowNavigationAid) && !within(Aspect) && !within(lopsil..*);
	pointcut srmCalls():
		call(* securityRelevantMethod(..));
	

	// handles lopsil reaction/action classes
	Object around(): notInAdvice() && srmCalls() {
		Object result = new Object();
		Action a = new Action(thisJoinPoint);
		Reaction r1 = p.react(a);
		a.setReaction(r1);

		// OK reaction
		if(r1.isOK()) {
			//Log.d("-->Aspect", "ok reaction: reaction received");		
			if(a.getReturnType().equals("void"))
				return proceed();
			else 
				result = proceed();	
			a.setResult(result);
			Reaction r2 = p.react(a);
			
			// OK reaction for result
			if(r2.isOK()) {
				//Log.d("-->Aspect", "Inner ok: reaction received, exiting");	
			    return result;
			}
			// REPLACE reaction for result
			else if(r2.isReplace()) {
				//Log.d("-->Aspect", "Inner replace: reaction received, exiting");	
				try {
					r1.checkReplVal(thisJoinPoint.getSignature().getDeclaringType());
				} 
				catch (LopsilException le) {
					Log.d("-->Aspect", "exception in inner replacement");
		    		System.exit(1);				    			
				}
				return r2.getReplVal();
			} 
			// HALT reaction for result
			else if(r2.isHalt()) {
				//Log.d("-->Aspect", "halting now inner");
	    		System.exit(1);
			} 
			// EXCEPTION reaction for result
			else {
				//Log.d("-->Aspect", "Inner Exception: reaction received, exiting");	
				try { 
					throw r2.getException();
				} 
				catch (Exception e) {
		    		System.exit(1);		
				}
			}
		} 
		// REPLACE reaction
		else if(r1.isReplace()) {
			//Log.d("-->Aspect", "replacing return value");
			try {
				r1.checkReplVal(thisJoinPoint.getSignature().getDeclaringType());
			} 
			catch (LopsilException le) {
				Log.d("-->Aspect", "exception while replacing initial");
	    		System.exit(1);				    			
		 	}
			return r1.getReplVal();
		} 
		// HALT reaction
		else if(r1.isHalt()) {
			//Log.d("-->Aspect", "halting now initial");
			System.exit(1);
		} 
		// EXCEPTION reaction
		else {
			//Log.d("-->Aspect", "Exception in initial");	
			try { 
				throw r1.getException();
			} 
			catch (Exception e) {
	    		System.exit(1);				
			}
		}
		return result;
	}
	
	@Override
	public void onProviderDisabled(String arg0) { }
	@Override
	public void onProviderEnabled(String arg0) { }
	@Override
	public void onStatusChanged(String arg0, int arg1, Bundle arg2) { }
}