package lopsil.device;
import lopsil.*;
import javax.swing.JOptionPane;

/**
 * FrequencyOfUpdatesAssumption checks to make sure that a GPS LocationDevice is updating
 * at least at a specific user-defined interval.
 * 
 * @author Billy Rickey
 * @author Jay Ligatti
 *
 */
public class FrequencyOfUpdatesAssumption implements Runnable{
	
	/* Instance Fields */
	private long _msec;
	private volatile LocationDevice[] _gd;
	private Policy _policy;
	
	public static final int MILSECONDS = 1;
	public static final int SECONDS = 2;
	public static final int MINUTES = 3;
	public static final int HOURS = 4;
	
	
	/**
	 * Construct a newly allocated FrequencyOfUpdatesAssumption.
	 */
	//public FrequencyOfUpdatesAssumption(){_msec = (long) (1.0 / 0.0);}
	
	/**
	 * Construct a newly allocated FrequencyOfUpdatesAssumption with a specified time
	 * @param time the time interval at which to check
	 * @param type the time measurement identifier
	 */
	public FrequencyOfUpdatesAssumption(double time, int type) {
		switch(type){
			case MILSECONDS:
				_msec = (long)time;
				break;
			case SECONDS:
				_msec = (long)time * 1000;
				break;
			case MINUTES:
				_msec = (long)time * 60000;
				break;
			case HOURS:
				_msec = (long)time * 3600000;
				break;
		//	case INFINITY:
			//	_msec = (long) (1.0 / 0.0);
		}
		//_delay = 2;
	}
	
	/**
	 * Construct a newly allocated FrequencyOfUpdatesAssumption with an existing FrequencyOfUpdatesAssumption
	 * and an array of LocationDevices
	 * @param t the existing FrequencyOfUpdatesAssumption
	 * @param locationDevice the LocationDevices to use the FrequencyOfUpdatesAssumption
	 */
	public FrequencyOfUpdatesAssumption(FrequencyOfUpdatesAssumption t, LocationDevice[] locationDevice, Policy p) {
		_msec = t.getMilliseconds();
		_policy = p;
		_gd = locationDevice;
	}
	
	/**
	 * Compare the location update time with the current system time to make sure the device
	 * is updating at within the specified interval.
	 */
	public void run(){
		while(true) {
			try {
				Thread.currentThread().sleep(_msec);
				for(int i = 0; i < _gd.length; i++) {
					if(_gd[i].isGPS()) {
						/*if(!_gd[i].isActive()) {
							JOptionPane.showConfirmDialog(null, "No communication with GPS data.  Please restart application.",
									"Communication Error", JOptionPane.PLAIN_MESSAGE);
							System.exit(1);
						}*/
						if(!_gd[i].isUpdated()) {
							
							try{
								_policy.handleTimeOfUpdateViolation();
							}catch(Exception ex) {
								if(JOptionPane.showConfirmDialog(null, ex.toString(), "Continue?", JOptionPane.YES_NO_OPTION) != 0)
									System.exit(1);									
							}
						}
						_gd[i].setNotUpdated();
					}
				}
			} catch (Exception e) {
				System.err.println(e);
				System.exit(1);
			}
		}
	}
	
	/**
	 * Return the milliseconds at which the device should be updating.
	 * @return the milliseconds
	 */
	public long getMilliseconds(){return _msec;}

} // end of FrequencyOfUpdatesAssumption