package lopsil.device;

import java.awt.event.*;
import javax.swing.*;
import lopsil.*;

/**
 * LopsilWindowDevice allows for a Window LocationDevice to be used with a specified set
 * of locations.
 * 
 * @author Billy Rickey
 * @author Jay Ligatti
 * @author Nalin Saigal
 */
public class LopsilWindowDevice extends LocationDevice {
	
	private JFrame box;
	private JRadioButton[] _buttons;
	private String[] _buttonTexts;
	private static Location _location;

	/**
	 * Create a newly allocated LopsilWindowDevice with an array of Strings to be used as
	 * selectable locations.
	 * @param buttonText an array of Strings to be used as selectable locations
	 */
	public LopsilWindowDevice(String[] buttonText){
		// Check for erroneous number of arguments
		try {
			checkLocations(buttonText.length);
		} catch(LopsilException e) {
			System.out.println(e);
			System.exit(1);
		}
		
		this._location = new Location();
		_buttonTexts = buttonText;
		_gps = false;
		_gran = null;
		_buttons = new JRadioButton[buttonText.length];
		
		box = new JFrame("LoPSiL Window");
		box.setSize(200, 100);
		box.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Setup each radio button
		for(int i = 0; i < buttonText.length; i++) {
			_buttons[i] = new JRadioButton(buttonText[i]);
			_buttons[i].setAlignmentX(JRadioButton.CENTER_ALIGNMENT);
			_buttons[i].addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					Boolean enabled = new Boolean(e.getStateChange() == ItemEvent.SELECTED);
					if(enabled) {
						for(int j = 0; j < _buttons.length; j++) {
							if(!_buttons[j].isEnabled()) {
								_buttons[j].setEnabled(enabled);
								_buttons[j].setSelected(false);
							} else if(_buttons[j].isSelected()) {
								_buttons[j].setSelected(enabled);
								_buttons[j].setEnabled(!enabled);
								LopsilWindowDevice.this._location.setWindowLocation(_buttonTexts[j]);
							}
						}
					}
				}
			});
		}
		_buttons[0].setSelected(true);
		LopsilWindowDevice.this._location.setWindowLocation(_buttonTexts[0]);
		box.setLayout(new BoxLayout(box.getContentPane(), BoxLayout.Y_AXIS));
		
		for(int i = 0; i < buttonText.length; i++)
			box.getContentPane().add(_buttons[i]);
		
		box.setResizable(false);
		box.pack();
		box.setLocation(850, 400);
		box.setVisible(true);
	}
	
	/**
	 * Ensures that the number of locations specified in for the
	 * construction of a LopsilWindowDevice is an appropriate amount.
	 * 
	 * @param number number of locations specified 
	 * @throws LopsilException
	 */
	private static void checkLocations(int number) throws LopsilException {
		if(number < 1) 
			throw new LopsilException("LopsilWindowDevice needs at least 1 location!");
		if(number > 10)
			throw new LopsilException("LopsilWindowDevice delcared with too many locations!");
	}
	
	/**
	 * Returns the Location of the LocationDevice
	 * @return the Location object of the LocationDevice
	 */
	public Location getLocation(){return this._location;}
	
	/**
	 * Sets the Location of the LocationDevice
	 * @param l the Location object to be set
	 */
	public void setLocation(Location l){this._location.setWindowLocation(l.getWindowLocation());}
	
} // end LopsilWindowDevice